# frozen_string_literal: true

# version.rb: Prawn icon versioning.
#
# Copyright October 2014, Jesse Doyle. All rights reserved.
#
# This is free software. Please see the LICENSE and COPYING files for details.

module Prawn
  class Icon
    VERSION = '4.1.0'
  end
end
